@@ -17,12 +17,12 @@
 //
 //-----------------------------------------------------------------------------
 
-#include <stdio.h>
 #include "config.h"
 
 #include "SDL.h" // haleyjd
 
 #include "i_printf.h"
+#include "i_system.h"
 #include "m_argv.h"
 #include "version.h"
 
@@ -34,6 +34,13 @@
 
 void D_DoomMain(void);
 
+#if defined(WIN_LAUNCHER)
+__declspec(dllexport) void Woof_Exit(void)
+{
+    I_SafeExit(0);
+}
+#endif
+
 #if defined(WIN_LAUNCHER)
 __declspec(dllexport) int Woof_Main(int argc, char **argv)
 #else
