@@ -23,6 +23,7 @@
 #include "../miniz/miniz.h"
 
 #include "doomstat.h"
+#include "i_printf.h"
 #include "v_video.h"
 #include "d_main.h"
 #include "st_stuff.h"
@@ -46,7 +47,7 @@ int NONWIDEWIDTH; // [crispy] non-widescreen SCREENWIDTH
 int WIDESCREENDELTA; // [crispy] horizontal widescreen offset
 
 boolean use_vsync;  // killough 2/8/98: controls whether vsync is called
-boolean hires, default_hires;      // killough 11/98
+int hires, default_hires;      // killough 11/98
 boolean use_aspect;
 boolean uncapped, default_uncapped; // [FG] uncapped rendering frame rate
 int fpslimit; // when uncapped, limit framerate to this value
@@ -578,8 +579,7 @@ static void I_InitDiskFlash(void)
   old_data = Z_Malloc((16<<hires) * (16<<hires) * sizeof(*old_data), PU_STATIC, 0);
 
   V_GetBlock(0, 0, 0, 16, 16, temp);
-  V_DrawPatchDirect(0-WIDESCREENDELTA, 0, 0, W_CacheLumpName(M_CheckParm("-cdrom") ?
-                                             "STCDROM" : "STDISK", PU_CACHE));
+  V_DrawPatchDirect(0-WIDESCREENDELTA, 0, 0, W_CacheLumpName("STDISK", PU_CACHE));
   V_GetBlock(0, 0, 0, 16, 16, diskflash);
   V_DrawBlock(0, 0, 0, 16, 16, temp);
 }
@@ -789,7 +789,7 @@ boolean I_WritePNGfile(char *filename)
         if (fwrite(png, 1, size, file) == size)
         {
           ret = true;
-          printf("I_WritePNGfile: %s\n", filename);
+          I_Printf(VB_INFO, "I_WritePNGfile: %s", filename);
         }
         fclose(file);
       }
@@ -827,8 +827,8 @@ static void CenterWindow(int *x, int *y, int w, int h)
 
     if (SDL_GetDisplayBounds(video_display, &bounds) < 0)
     {
-        fprintf(stderr, "CenterWindow: Failed to read display bounds "
-                        "for display #%d!\n", video_display);
+        I_Printf(VB_WARNING, "CenterWindow: Failed to read display bounds "
+                             "for display #%d!", video_display);
         return;
     }
 
@@ -849,10 +849,10 @@ static void I_ResetInvalidDisplayIndex(void)
     // and if it doesn't, reset it.
     if (video_display < 0 || video_display >= SDL_GetNumVideoDisplays())
     {
-        fprintf(stderr,
+        I_Printf(VB_WARNING,
                 "I_ResetInvalidDisplayIndex: We were configured to run on "
                 "display #%d, but it no longer exists (max %d). "
-                "Moving to display 0.\n",
+                "Moving to display 0.",
                 video_display, SDL_GetNumVideoDisplays() - 1);
         video_display = 0;
     }
