@@ -17,6 +17,7 @@
 
 
 #include "doomtype.h"
+#include "i_printf.h"
 #include <stdio.h>
 #include "midifile.h"
 #include "midifallback.h"
@@ -112,9 +113,9 @@ static boolean GetProgramFallback(byte idx, byte program,
             // selected this variation or if the variation is above 63.
             fallback->value = 0;
 
-            fprintf(stderr,
+            I_Printf(VB_WARNING,
                     "midifallback: warning: ch=%d [bank_msb=%d prog=%d] "
-                    "falling back to [bank_msb=%d prog=%d]\n",
+                    "falling back to [bank_msb=%d prog=%d]",
                     idx, bank_msb[idx], program, fallback->value, program);
             return true;
         }
@@ -132,9 +133,9 @@ static boolean GetProgramFallback(byte idx, byte program,
             fallback->value -= 8;
         }
 
-        fprintf(stderr,
+        I_Printf(VB_WARNING,
                 "midifallback: warning: ch=%d [bank_msb=%d prog=%d] "
-                "falling back to [bank_msb=%d prog=%d]\n",
+                "falling back to [bank_msb=%d prog=%d]",
                 idx, bank_msb[idx], program, fallback->value, program);
         return true;
     }
@@ -149,9 +150,9 @@ static boolean GetProgramFallback(byte idx, byte program,
             fallback->value = drums_table[program];
             selected[idx] = true;
 
-            fprintf(stderr,
+            I_Printf(VB_WARNING,
                     "midifallback: warning: ch=%d [prog=%d] "
-                    "falling back to [prog=%d] (drums)\n",
+                    "falling back to [prog=%d] (drums)",
                     idx, program, fallback->value);
             return true;
         }
@@ -193,9 +194,9 @@ void MIDI_CheckFallback(const midi_event_t *event, midi_fallback_t *fallback,
                         fallback->type = FALLBACK_BANK_LSB;
                         fallback->value = 0;
 
-                        fprintf(stderr,
+                        I_Printf(VB_WARNING,
                                 "midifallback: warning: ch=%d [bank_lsb=%d] "
-                                "replaced by [bank_lsb=%d]\n",
+                                "replaced by [bank_lsb=%d]",
                                 idx, event->data.channel.param2, fallback->value);
                         return;
                     }
