@@ -19,6 +19,7 @@
 //-----------------------------------------------------------------------------
 
 #include "doomstat.h"
+#include "i_printf.h"
 #include "r_main.h"
 #include "p_mobj.h"
 #include "p_maputl.h"
@@ -109,7 +110,7 @@ static boolean PIT_StompThing (mobj_t *thing)
   if (!telefrag)  // killough 8/9/98: make consistent across all levels
     return false;
 
-  P_DamageMobj (thing, tmthing, tmthing, 10000); // Stomp!
+  P_DamageMobjBy (thing, tmthing, tmthing, 10000, MOD_Telefrag); // Stomp!
 
   return true;
 }
@@ -455,7 +456,7 @@ static boolean PIT_CheckLine(line_t *ld) // killough 3/26/98: make static
 	  if (numspechit == MAXSPECIALCROSS_ORIGINAL + 1)
 	  {
 	    overflow[emu_spechits].triggered = true;
-	    fprintf(stderr, "PIT_CheckLine: Triggered SPECHITS overflow!\n");
+	    I_Printf(VB_WARNING, "PIT_CheckLine: Triggered SPECHITS overflow!");
 	  }
 	  SpechitOverrun(ld);
 	}
@@ -565,7 +566,7 @@ static boolean PIT_CheckThing(mobj_t *thing) // killough 3/26/98: make static
 				      !(tmthing->flags & MF_SOLID)))
     {
       // [crispy] mobj or actual sprite height
-      const fixed_t thingheight = (CRITICAL(direct_vertical_aiming) && tmthing->target && tmthing->target->player) ?
+      const fixed_t thingheight = (direct_vertical_aiming && tmthing->target && tmthing->target->player) ?
         thing->info->actualheight : thing->height;
 
       // see if it went over / under
@@ -943,6 +944,7 @@ boolean P_TryMove(mobj_t *thing, fixed_t x, fixed_t y, boolean dropoff)
 static boolean PIT_ApplyTorque(line_t *ld)
 {
   if (ld->backsector &&       // If thing touches two-sided pivot linedef
+      (ld->dx || ld->dy) && // Torque is undefined if the line has no length
       tmbbox[BOXRIGHT]  > ld->bbox[BOXLEFT]  &&
       tmbbox[BOXLEFT]   < ld->bbox[BOXRIGHT] &&
       tmbbox[BOXTOP]    > ld->bbox[BOXBOTTOM] &&
@@ -1572,7 +1574,7 @@ static boolean PTR_ShootTraverse(intercept_t *in)
       // [crispy] check if the pullet puff's z-coordinate is below or above
       // its spawning sector's floor or ceiling, respectively, and move its
       // coordinates to the point where the trajectory hits the plane
-      if (CRITICAL(direct_vertical_aiming) && aimslope)
+      if (direct_vertical_aiming && aimslope)
       {
         const int lineside = P_PointOnLineSide(x, y, li);
         int side;
@@ -1615,7 +1617,7 @@ static boolean PTR_ShootTraverse(intercept_t *in)
   dist = FixedMul (attackrange, in->frac);
   {
   // [crispy] mobj or actual sprite height
-  const fixed_t thingheight = (CRITICAL(direct_vertical_aiming) && shootthing->player) ?
+  const fixed_t thingheight = (direct_vertical_aiming && shootthing->player) ?
     th->info->actualheight : th->height;
   thingtopslope = FixedDiv (th->z+thingheight - shootz , dist);
   }
@@ -1672,9 +1674,9 @@ fixed_t P_AimLineAttack(mobj_t *t1,angle_t angle,fixed_t distance,int mask)
 
   // can't shoot outside view angles
 
-  if (t1->player && CRITICAL(direct_vertical_aiming))
+  if (t1->player && direct_vertical_aiming)
   {
-    bottomslope = (topslope = PLAYER_SLOPE(t1->player) + 1) - 2;
+    bottomslope = (topslope = t1->player->slope + 1) - 2;
   }
   else
   {
@@ -1962,7 +1964,7 @@ boolean PIT_ChangeSector(mobj_t *thing)
     {
       int t;         // killough 8/10/98
 
-      P_DamageMobj(thing,NULL,NULL,10);
+      P_DamageMobjBy(thing,NULL,NULL,10,MOD_Crush);
 
       // spray blood in a random direction
       mo = P_SpawnMobj (thing->x,
@@ -2383,9 +2385,9 @@ static void SpechitOverrun(line_t *ld)
             nofit = addr;
             break;
         default:
-            fprintf(stderr, "SpechitOverrun: Warning: unable to emulate"
-                            "an overrun where numspechit=%i\n",
-                            numspechit);
+            I_Printf(VB_DEBUG, "SpechitOverrun: Warning: unable to emulate"
+                               "an overrun where numspechit=%i",
+                               numspechit);
             break;
     }
 }
