@@ -77,6 +77,16 @@ void A_Recoil(player_t* player)
 static void P_SetPsprite(player_t *player, int position, statenum_t stnum)
 {
   P_SetPspritePtr(player, &player->psprites[position], stnum);
+
+  if (position == ps_weapon)
+  {
+    const weaponinfo_t wp = weaponinfo[player->readyweapon];
+
+    if (stnum == wp.upstate)
+      player->switching = weapswitch_raising;
+    else if (stnum == wp.downstate)
+      player->switching = weapswitch_lowering;
+  }
 }
 
 //
@@ -153,7 +163,6 @@ static void P_BringUpWeapon(player_t *player)
     WEAPONBOTTOM+FRACUNIT*2 : WEAPONBOTTOM;
 
   P_SetPsprite(player, ps_weapon, newstate);
-  player->switching = weapswitch_raising;
 }
 
 // The first set is where the weapon preferences from             // killough,
@@ -435,7 +444,6 @@ static void P_FireWeapon(player_t *player)
 void P_DropWeapon(player_t *player)
 {
   P_SetPsprite(player, ps_weapon, weaponinfo[player->readyweapon].downstate);
-  player->switching = weapswitch_lowering;
 }
 
 //
@@ -477,7 +485,6 @@ void A_WeaponReady(player_t *player, pspdef_t *psp)
       // change weapon (pending weapon should already be validated)
       statenum_t newstate = weaponinfo[player->readyweapon].downstate;
       P_SetPsprite(player, ps_weapon, newstate);
-      player->switching = weapswitch_lowering;
       return;
     }
   else
@@ -786,9 +793,9 @@ void A_FireOldBFG(player_t *player, pspdef_t *psp)
 	  // killough 8/2/98: make autoaiming prefer enemies
 	  int mask = MF_FRIEND;
 	  fixed_t slope;
-	  if (CRITICAL(direct_vertical_aiming))
+	  if (direct_vertical_aiming)
 	  {
-	    slope = PLAYER_SLOPE(mo->player);
+	    slope = mo->player->slope;
 	  }
 	  else
 	  do
@@ -856,9 +863,9 @@ static void P_BulletSlope(mobj_t *mo)
   // killough 8/2/98: make autoaiming prefer enemies
   int mask = demo_version < 203 ? 0 : MF_FRIEND;
 
-  if (CRITICAL(direct_vertical_aiming))
+  if (direct_vertical_aiming)
   {
-    bulletslope = PLAYER_SLOPE(mo->player);
+    bulletslope = mo->player->slope;
   }
   else
   do
