@@ -235,6 +235,7 @@ void P_MovePlayer (player_t* player)
     player->lookdir = BETWEEN(-LOOKDIRMAX * MLOOKUNIT,
                                LOOKDIRMAX * MLOOKUNIT,
                                player->lookdir + cmd->lookdir);
+    player->slope = PLAYER_SLOPE(player);
   }
 }
 
@@ -404,6 +405,7 @@ void P_PlayerThink (player_t* player)
 
   if (player->playerstate == PST_DEAD)
     {
+      player->slope = PLAYER_SLOPE(player); // For 3D audio pitch angle.
       P_DeathThink (player);
       return;
     }
@@ -522,13 +524,13 @@ void P_PlayerThink (player_t* player)
   if (player->powers[pw_ironfeet] > 0)        // killough
     player->powers[pw_ironfeet]--;
 
-  if (player->powers[pw_renderstats])
+  if (player->cheats & CF_RENDERSTATS)
   {
     extern void R_ShowRenderingStats();
     R_ShowRenderingStats();
   }
 
-  if (player->powers[pw_mapcoords])
+  if (player->cheats & CF_MAPCOORDS)
   {
     extern void cheat_mypos_print();
     cheat_mypos_print();
