@@ -294,6 +294,18 @@ static void R_MaybeInterpolateSector(sector_t* sector)
         {
             sector->interpceilingheight = sector->ceilingheight;
         }
+
+        if (sector->oldscrollgametic == gametic - 1)
+        {
+            sector->floor_xoffs = sector->old_floor_xoffs +
+                FixedMul(sector->base_floor_xoffs - sector->old_floor_xoffs, fractionaltic);
+            sector->floor_yoffs = sector->old_floor_yoffs +
+                FixedMul(sector->base_floor_yoffs - sector->old_floor_yoffs, fractionaltic);
+            sector->ceiling_xoffs = sector->old_ceiling_xoffs +
+                FixedMul(sector->base_ceiling_xoffs - sector->old_ceiling_xoffs, fractionaltic);
+            sector->ceiling_yoffs = sector->old_ceiling_yoffs +
+                FixedMul(sector->base_ceiling_yoffs - sector->old_ceiling_yoffs, fractionaltic);
+        }
     }
     else
     {
@@ -302,6 +314,17 @@ static void R_MaybeInterpolateSector(sector_t* sector)
     }
 }
 
+static void R_MaybeInterpolateTextureOffsets(side_t *side)
+{
+    if (uncapped && side->oldgametic == gametic - 1)
+    {
+        side->textureoffset = side->oldtextureoffset +
+            FixedMul(side->basetextureoffset - side->oldtextureoffset, fractionaltic);
+        side->rowoffset = side->oldrowoffset +
+            FixedMul(side->baserowoffset - side->oldrowoffset, fractionaltic);
+    }
+}
+
 //
 // R_AddLine
 // Clips the given segment
@@ -372,6 +395,8 @@ static void R_AddLine (seg_t *line)
   if (x1 >= x2)       // killough 1/31/98 -- change == to >= for robustness
     return;
 
+  R_MaybeInterpolateTextureOffsets(line->sidedef);
+
   backsector = line->backsector;
 
   // Single sided line?
