@@ -338,6 +338,7 @@ static void do_draw_plane(visplane_t *pl)
       {
 	int texture;
 	angle_t an, flip;
+	boolean vertically_scrolling = false;
 
 	// killough 10/98: allow skies to come from sidedefs.
 	// Allows scrolling and/or animated skies, as well as
@@ -354,6 +355,9 @@ static void do_draw_plane(visplane_t *pl)
 	    // Sky transferred from first sidedef
 	    const side_t *s = *l->sidenum + sides;
 
+	    if (s->baserowoffset - s->oldrowoffset)
+	      vertically_scrolling = true;
+
 	    // Texture comes from upper texture of reference sidedef
 	    texture = texturetranslation[s->toptexture];
 
@@ -401,7 +405,7 @@ static void do_draw_plane(visplane_t *pl)
           dc_texturemid = dc_texturemid * dc_texheight / SKYSTRETCH_HEIGHT;
           colfunc = R_DrawColumn;
         }
-        else
+        else if (!vertically_scrolling)
         {
           // Make sure the fade-to-color effect doesn't happen too early
           fixed_t diff = dc_texturemid - ORIGHEIGHT / 2 * FRACUNIT;
@@ -419,8 +423,8 @@ static void do_draw_plane(visplane_t *pl)
         for (x = pl->minx; (dc_x = x) <= pl->maxx; x++)
           if ((dc_yl = pl->top[x]) != USHRT_MAX && dc_yl <= (dc_yh = pl->bottom[x]))
             {
-              dc_source = R_GetColumn(texture, ((an + xtoskyangle[x])^flip) >>
-				      ANGLETOSKYSHIFT);
+              dc_source = R_GetColumnMod2(texture, ((an + xtoskyangle[x])^flip) >>
+				         ANGLETOSKYSHIFT);
               colfunc();
             }
 
